import pytest
from rest_framework import status
from history4feed.app.models import Post, Feed, Job
from datetime import datetime as dt
from unittest.mock import patch
from history4feed.app.serializers import PostWithFeedIDSerializer
from history4feed.app.views import PostOnlyView
from dateutil.parser import parse as parse_date


from history4feed.app.utils import (
    DatetimeFieldUTC,
    Ordering,
    Pagination,
    MinMaxDateFilter,
)

# from .openapi_params import FEED_PARAMS, POST_PARAMS

from history4feed.app.serializers import PostWithFeedIDSerializer
from history4feed.app.models import Post, Feed, Job
from rest_framework import (
    status,
)
from django_filters.rest_framework import (
    DjangoFilterBackend,
)


@pytest.mark.parametrize(
    "data",
    [
        dict(title="New title", pubdate="2024-01-01"),
        dict(pubdate="2021-01-01", author="new author 1"),
        dict(categories=['cat1', 'cat2'], author="new author 2"),
        dict(categories=['cat-1', 'cat0'], author="new author 3", pubdate='2025-01-01T10:59:01Z'),
    ]
)
@pytest.mark.django_db
def test_partial_update_post(client, data):
    feed = Feed.objects.create(title="test feed", url="https://example.com/")
    post = Post.objects.create(feed=feed, title="Old title", pubdate=dt.now())

    url = f"/api/v1/posts/{post.id}/"

    response = client.patch(url, data, content_type="application/json")

    assert response.status_code == status.HTTP_201_CREATED
    post.refresh_from_db()
    if title := data.get('title'):
        assert post.title == title
    if author := data.get('author'):
        assert post.author == author
    if categories := data.get('categories'):
        assert set([obj.name for obj in post.categories.all()]) == set(categories)
    if pubdate := data.get('pubdate'):
        assert post.pubdate == DatetimeFieldUTC().to_python(parse_date(pubdate))


@pytest.mark.django_db
def test_destroy_post(client):
    feed = Feed.objects.create(title="feed destroy test", url="https://example.com/")
    post = Post.objects.create(feed=feed, title="to be deleted", pubdate=dt.now())

    url = f"/api/v1/posts/{post.id}/"

    response = client.delete(url)
    assert response.status_code == status.HTTP_204_NO_CONTENT

    post.refresh_from_db()
    assert post.deleted_manually is True

    response = client.get(url)
    assert (
        response.status_code == status.HTTP_404_NOT_FOUND
    ), "should already be deleted"


@pytest.mark.django_db
def test_reindex_post(client):
    feed = Feed.objects.create(title="Reindex Test Feed", url="https://example.com/")
    post = Post.objects.create(feed=feed, title="Reindex Me", pubdate=dt.now())

    mock_job = Job.objects.create(state="pending", feed=feed)

    with patch(
        "history4feed.app.views.task_helper.new_patch_posts_job", return_value=mock_job
    ) as mock_task:
        url = f"/api/v1/posts/{post.id}/reindex/"
        data = {}

        response = client.patch(url, data, content_type="application/json")

        assert response.status_code == status.HTTP_201_CREATED
        assert "id" in response.json()
        assert response.json()["id"] == str(mock_job.id)
        assert response.json()["feed_id"] == str(feed.id)

        mock_task.assert_called_once_with(feed, [post])


@pytest.mark.django_db
def test_list_posts(client):
    feed = Feed.objects.create(title="list test feed", url="https://example.com/")
    Post.objects.create(feed=feed, title="First post", pubdate=dt.now())
    Post.objects.create(
        feed=feed,
        title="Second post",
        pubdate=dt.now(),
        link="https://example.net/post2",
    )

    with patch(
        "history4feed.app.views.PostOnlyView.filter_queryset", side_effect=lambda qs: qs
    ) as mock_filter:

        url = "/api/v1/posts/"
        response = client.get(url)

        assert response.status_code == status.HTTP_200_OK
        assert len(response.json()["posts"]) == 2  # Pagination applied
        mock_filter.assert_called_once()


@pytest.mark.django_db
def test_retrieve_post(client):
    feed = Feed.objects.create(title="retrieve feed", url="https://example.com/")
    post = Post.objects.create(feed=feed, title="Single Post", pubdate=dt.now())

    url = f"/api/v1/posts/{post.id}/"
    response = client.get(url)

    assert response.status_code == status.HTTP_200_OK
    assert response.json()["id"] == str(post.id)


def test_class_variables():
    assert PostOnlyView.ordering_fields == [
        "pubdate",
        "title",
        "datetime_updated",
        "datetime_added",
    ]
    assert PostOnlyView.ordering == "pubdate_descending", "default ordering"
    assert PostOnlyView.filter_backends == [
        DjangoFilterBackend,
        Ordering,
        MinMaxDateFilter,
    ]
    assert type(PostOnlyView.pagination_class) == Pagination
    assert PostOnlyView.pagination_class.results_key == "posts"
    assert PostOnlyView.serializer_class == PostWithFeedIDSerializer



from itertools import tee
from operator import lt
import os
import time
from types import SimpleNamespace
import unittest, pytest
from urllib.parse import urljoin

base_url = os.environ["SERVICE_BASE_URL"]
import requests



def remove_unknown_keys(data: dict, known_keys: list):
    payload = data.copy()
    for k in list(payload.keys()):
        if k not in known_keys:
            payload.pop(k, None)
    return payload


def wait_for_jobs(job_id):
    try_count = 0
    while True:
        job_data = requests.get(f"{base_url}/api/v1/jobs/{job_id}/").json()
        job_status = job_data["state"]
        if job_status in ["success", "failed"]:
            assert job_status == "success", f"response: {job_data}"
            return job_data
        try_count += 1
        assert try_count < 30, "stopped after 30 retries"
        time.sleep(3)


def get_post_ids_for_job(job:dict):
    retval = []
    for type, d in job['urls'].items():
        if type == 'skipped':
            continue
        for p in d:
            retval.append((p['id'], type))
    return retval
    

def is_sorted(iterable, key=None, reverse=False):
    it = iterable if (key is None) else map(key, iterable)
    a, b = tee(it)
    next(b, None)
    if reverse:
        b, a = a, b
    return not any(map(lt, b, a))


from datetime import UTC, datetime
import os
import time
from types import SimpleNamespace
import unittest, pytest
from urllib.parse import urljoin
from dateutil.parser import parse as parse_date

from tests.utils import remove_unknown_keys, wait_for_jobs

base_url = os.environ["SERVICE_BASE_URL"]
import requests
@pytest.mark.parametrize(
        ["post_id", "metadata"],
        [
            ["58514345-4e10-54c9-8f2c-d81507088079", dict(title="updated post title")],
            ["a378c839-0940-56fb-b52c-e5b78d34ec94", dict(title="updated title", author="new post author")],
            ["58514345-4e10-54c9-8f2c-d81507088079", dict(pubdate="2009-03-04T14:56:07Z")],
        ]
)
def test_update_post_metadata(post_id, metadata):
    resp = requests.patch(urljoin(base_url, f"api/v1/posts/{post_id}/"), json=metadata)
    assert resp.status_code == 201
    resp_data = resp.json()

    if expected_pretty_url := metadata.get("pretty_url"):
        assert resp_data["pretty_url"] == expected_pretty_url

    if expected_categories := metadata.get("categories"):
        assert resp_data["categories"] == expected_categories

    if expected_author := metadata.get("author"):
        assert resp_data["author"] == expected_author

    
    if expected_pubdate := metadata.get("pubdate"):
        assert resp_data["pubdate"] == expected_pubdate


from unittest.mock import call, patch

import pytest
from history4feed.app.models import Feed, Post
from datetime import datetime as dt

from history4feed.h4fscripts.build_rss import build_entry_element, build_rss


@pytest.mark.django_db
def test_create_rss():
    feed = Feed.objects.create(url="https://example.com/rss.xml", title="Test Feed")
    posts = [
        Post.objects.create(
            feed=feed,
            title="First post",
            pubdate=dt.now(),
            link="http://example.com/post/1/",
        ),
        Post.objects.create(
            feed=feed,
            title="First post 2",
            pubdate=dt.now(),
            link="http://example.com/post/3/",
            author="some author",
        ),
        Post.objects.create(
            feed=feed,
            title="Second post",
            pubdate=dt.now(),
            link="https://example.net/post2",
        ),

        Post.objects.create(
            feed=feed,
            title="Third post with description",
            description="The description",
            pubdate=dt.now(),
            link="http://example.com/post/6/",
        ),
    ]
    posts[2].add_categories(["cat1"])

    with patch(
        "history4feed.h4fscripts.build_rss.build_entry_element",
        side_effect=build_entry_element,
    ) as mock_build_post_entry:

        build_rss(feed, posts)
        d = mock_build_post_entry.call_args[0][1]
        mock_build_post_entry.assert_has_calls(
            [call(post, d) for post in posts], any_order=True
        )



rss_example = """
<rss xmlns:atom="http://www.w3.org/2005/Atom" version="2.0">
    <channel>
        <title>Your awesome title</title>
        <description>RSS -- Contains encoded html -- PARTIAL CONTENT ONLY</description>
        <link>https://muchdogesec.github.io/fakeblog123/</link>
        <item>
            <title>Obstracts AI relationship generation test 2</title>
            <description>Also useful for Obstracts testing</description>
            <pubDate>Sun, 01 Sep 2024 08:00:00 +0000</pubDate>
            <link>
                https://muchdogesec.github.io/fakeblog123///test1/2024/09/01/same-ioc-across-posts.html</link>
            <guid isPermaLink="true">
                https://muchdogesec.github.io/fakeblog123///test1/2024/09/01/same-ioc-across-posts.html</guid>
            <author>David G</author>
            <category>Blog</category>
            <category>Test</category>
        </item>
        <item>
            <title>Obstracts AI relationship generation test</title>
            <description>Useful for Obstracts testing</description>
            <pubDate>Fri, 23 Aug 2024 08:00:00 +0000</pubDate>
            <link>
                https://muchdogesec.github.io/fakeblog123///test1/2024/08/23/obstracts-ai-relationships.html</link>
            <guid isPermaLink="true">
                https://muchdogesec.github.io/fakeblog123///test1/2024/08/23/obstracts-ai-relationships.html</guid>
            <author>David G</author>
            <category>Blog</category>
            <category>Test</category>
        </item>
        <item>
            <title>Update this post for testing updates to posts</title>
            <description>Mainly the patch endpoint on h4f</description>
            <pubDate>Tue, 20 Aug 2024 10:00:00 +0000</pubDate>
            <link>https://muchdogesec.github.io/fakeblog123///test3/2024/08/20/update-to-post-1.html</link>
            <guid isPermaLink="true">
                https://muchdogesec.github.io/fakeblog123///test3/2024/08/20/update-to-post-1.html</guid>
            <author>David G</author>
            <category>Blog</category>
            <category>Test</category>
        </item>
        <item>
            <title>Testing Extractions 1</title>
            <description>lots of iocs</description>
            <pubDate>Wed, 07 Aug 2024 08:00:00 +0000</pubDate>
            <link>
                https://muchdogesec.github.io/fakeblog123///test2/2024/08/07/testing-extractions-1.html</link>
            <guid isPermaLink="true">
                https://muchdogesec.github.io/fakeblog123///test2/2024/08/07/testing-extractions-1.html</guid>
            <author>David G</author>
            <category>Blog</category>
            <category>Test</category>
        </item>
        <item>
            <title>Testing Markdown Elements</title>
            <description>Nearly all Markdown applications support the basic syntax outlined in the
                original Markdown design document.</description>
            <pubDate>Mon, 05 Aug 2024 08:00:00 +0000</pubDate>
            <link>
                https://muchdogesec.github.io/fakeblog123///test2/2024/08/05/testing-markdown-elements-1.html</link>
            <guid isPermaLink="true">
                https://muchdogesec.github.io/fakeblog123///test2/2024/08/05/testing-markdown-elements-1.html</guid>
            <author>David G</author>
            <category>Blog</category>
            <category>Test</category>
        </item>
        <item>
            <title>Real Post Example - Fighting Ursa Luring Targets With Car for Sale</title>
            <description>A Russian threat actor we track as Fighting Ursa.</description>
            <pubDate>Thu, 01 Aug 2024 08:00:00 +0000</pubDate>
            <link>
                https://muchdogesec.github.io/fakeblog123///test1/2024/08/01/real-post-example-1.html</link>
            <guid isPermaLink="true">
                https://muchdogesec.github.io/fakeblog123///test1/2024/08/01/real-post-example-1.html</guid>
            <author>David G</author>
            <category>Blog</category>
            <category>Test</category>
        </item>
    </channel>
</rss>
"""

atom_example = """
<feed xmlns="http://www.w3.org/2005/Atom">
    <generator uri="https://jekyllrb.com/" version="4.3.3">Jekyll</generator>
    <link href="https://muchdogesec.github.io/fakeblog123//feeds/atom-feed-cdata-partial.xml"
        rel="self" type="application/atom+xml" />
    <link href="https://muchdogesec.github.io/fakeblog123/" rel="alternate" type="text/html" />
    <updated>2024-08-27T10:17:18+01:00</updated>
    <id>https://muchdogesec.github.io/fakeblog123//feed.xml</id>
    <title type="html">Your awesome title</title>
    <subtitle>ATOM -- Contains decoded html inside CDATA tags -- PARTIAL CONTENT ONLY</subtitle>
    <author>
        <name>David G</name>
    </author>
    <entry>
        <link
            href="https://muchdogesec.github.io/fakeblog123///test1/2024/09/01/same-ioc-across-posts.html"
            rel="alternate" type="text/html" title="Obstracts AI relationship generation test 2" />
        <published>Sun, 01 Sep 2024 08:00:00 +0000</published>
        <updated>Sun, 01 Sep 2024 08:00:00 +0000</updated>
        <id>https://muchdogesec.github.io/fakeblog123///test1/2024/09/01/same-ioc-across-posts.html</id>
        <title>
    <![CDATA[ Obstracts AI relationship generation test 2 ]]>
    </title>
        <summary>
    <![CDATA[ Also useful for Obstracts testing ]]>
    </summary>
        <content>
    <![CDATA[ Also useful for Obstracts testing ]]>
    </content>
        <author>
            <name>David G</name>
            <name>G David</name>
        </author>
        <category term="Blog" />
        <category term="Test" />
    </entry>
    <entry>
        <link
            href="https://muchdogesec.github.io/fakeblog123///test1/2024/08/23/obstracts-ai-relationships.html"
            rel="alternate" type="text/html" title="Obstracts AI relationship generation test" />
        <published>Fri, 23 Aug 2024 08:00:00 +0000</published>
        <updated>Fri, 23 Aug 2024 08:00:00 +0000</updated>
        <id>
            https://muchdogesec.github.io/fakeblog123///test1/2024/08/23/obstracts-ai-relationships.html</id>
        <title>
    <![CDATA[ Obstracts AI relationship generation test ]]>
    </title>
        <summary>
    <![CDATA[ Useful for Obstracts testing ]]>
    </summary>
        <content>
    <![CDATA[ Useful for Obstracts testing ]]>
    </content>
        <author>
            <name>David G</name>
            <name>G David</name>
        </author>
        <category term="Blog" />
        <category term="Test" />
    </entry>
    <entry>
        <link
            href="https://muchdogesec.github.io/fakeblog123///test3/2024/08/20/update-to-post-1.html"
            rel="alternate" type="text/html" title="Update this post for testing updates to posts" />
        <published>Tue, 20 Aug 2024 10:00:00 +0000</published>
        <updated>Tue, 20 Aug 2024 10:00:00 +0000</updated>
        <id>https://muchdogesec.github.io/fakeblog123///test3/2024/08/20/update-to-post-1.html</id>
        <title>
    <![CDATA[ Update this post for testing updates to posts ]]>
    </title>
        <summary>
    <![CDATA[ Mainly the patch endpoint on h4f ]]>
    </summary>
        <content>
    <![CDATA[ Mainly the patch endpoint on h4f ]]>
    </content>
        <author>
            <name>David G</name>
            <name>G David</name>
        </author>
        <category term="Blog" />
        <category term="Test" />
    </entry>
    <entry>
        <link
            href="https://muchdogesec.github.io/fakeblog123///test2/2024/08/07/testing-extractions-1.html"
            rel="alternate" type="text/html" title="Testing Extractions 1" />
        <published>Wed, 07 Aug 2024 08:00:00 +0000</published>
        <updated>Wed, 07 Aug 2024 08:00:00 +0000</updated>
        <id>https://muchdogesec.github.io/fakeblog123///test2/2024/08/07/testing-extractions-1.html</id>
        <title>
    <![CDATA[ Testing Extractions 1 ]]>
    </title>
        <summary>
    <![CDATA[ lots of iocs ]]>
    </summary>
        <content>
    <![CDATA[ lots of iocs ]]>
    </content>
        <author>
            <name>David G</name>
            <name>G David</name>
        </author>
        <category term="Blog" />
        <category term="Test" />
    </entry>
    <entry>
        <link
            href="https://muchdogesec.github.io/fakeblog123///test2/2024/08/05/testing-markdown-elements-1.html"
            rel="alternate" type="text/html" title="Testing Markdown Elements" />
        <published>Mon, 05 Aug 2024 08:00:00 +0000</published>
        <updated>Mon, 05 Aug 2024 08:00:00 +0000</updated>
        <id>
            https://muchdogesec.github.io/fakeblog123///test2/2024/08/05/testing-markdown-elements-1.html</id>
        <title>
    <![CDATA[ Testing Markdown Elements ]]>
    </title>
        <summary>
    <![CDATA[ Nearly all Markdown applications support the basic syntax outlined in the original Markdown design document. ]]>
    </summary>
        <content>
    <![CDATA[ Nearly all Markdown applications support the basic syntax outlined in the original Markdown design document. ]]>
    </content>
        <author>
            <name>David G</name>
            <name>G David</name>
        </author>
        <category term="Blog" />
        <category term="Test" />
    </entry>
    <entry>
        <link
            href="https://muchdogesec.github.io/fakeblog123///test1/2024/08/01/real-post-example-1.html"
            rel="alternate" type="text/html"
            title="Real Post Example - Fighting Ursa Luring Targets With Car for Sale" />
        <published>Thu, 01 Aug 2024 08:00:00 +0000</published>
        <updated>Thu, 01 Aug 2024 08:00:00 +0000</updated>
        <id>https://muchdogesec.github.io/fakeblog123///test1/2024/08/01/real-post-example-1.html</id>
        <title>
    <![CDATA[ Real Post Example - Fighting Ursa Luring Targets With Car for Sale ]]>
    </title>
        <summary>
    <![CDATA[ A Russian threat actor we track as Fighting Ursa. ]]>
    </summary>
        <content>
    <![CDATA[ A Russian threat actor we track as Fighting Ursa. ]]>
    </content>
        <author>
            <name>David G</name>
            <name>G David</name>
        </author>
        <category term="Blog" />
        <category term="Test" />
    </entry>
</feed>
"""




import os
import time
from types import SimpleNamespace
import unittest, pytest
from urllib.parse import urljoin

base_url = os.environ["SERVICE_BASE_URL"]
import requests


def get_all_feeds():
    if not os.getenv('DELETE_ALL_FEEDS'):
        return []
    resp = requests.get(urljoin(base_url, "api/v1/feeds/"))
    return [[feed["id"]] for feed in resp.json()["feeds"]]

@pytest.mark.parametrize(
        ["feed_id"],
        get_all_feeds(),
)
def test_delete_blog(feed_id):
    resp = requests.delete(urljoin(base_url, f"api/v1/feeds/{feed_id}/"))
    assert resp.status_code == 204, "unexpected status code"
    resp = requests.get(urljoin(base_url, f"api/v1/feeds/{feed_id}/"))
    assert resp.status_code == 404, "feed should not exist after deletion"



from rest_framework import pagination, response, renderers
from rest_framework.filters import OrderingFilter, BaseFilterBackend
from django.utils.encoding import force_str
from django.db.models import Q
from datetime import datetime, UTC
import typing
from dogesec_commons.utils import Pagination, Ordering
from django.utils import timezone
from django.forms import DateTimeField
from django_filters.rest_framework import filters

class DatetimeFieldUTC(DateTimeField):
    def to_python(self, value):
        value = super().to_python(value) 
        return value and value.astimezone(UTC)
    
class DatetimeFilter(filters.Filter):
    field_class = DatetimeFieldUTC

class MinMaxDateFilter(BaseFilterBackend):
    min_val = datetime.min
    max_value = datetime.max
    def get_fields(self, view):
        out = {}
        fields = getattr(view, 'minmax_date_fields', [])
        if not isinstance(fields, list):
            return out
        for field in fields:
            out[f"{field}_max"] = field
            out[f"{field}_min"] = field
        return out
    
    def parse_date(self, value):
        return DatetimeFieldUTC().to_python(value)

    def filter_queryset(self, request, queryset, view):
        valid_fields = self.get_fields(view)
        valid_params = [(k, v) for k, v in request.query_params.items() if k in valid_fields]
        queries =  {}
        for param, value in valid_params:
            field_name = valid_fields[param]
            if param.endswith('_max'):
                queries[f"{field_name}__lte"] = self.parse_date(value)
            else:
                queries[f"{field_name}__gte"] = self.parse_date(value)
        return queryset.filter(**queries)

    def get_schema_operation_parameters(self, view):
        parameters = []
        valid_fields = self.get_fields(view)
        for query_name, field_name in valid_fields.items():
            _type = "Maximum"
            if query_name.endswith('min'):
                _type = "Minimum"
            parameter = {
                'name': query_name,
                'required': False,
                'in': 'query',
                'description': f"{_type} value of `{field_name}` to filter by in format `YYYY-MM-DD`.",
                'schema': {
                    'type': 'string', 'format': 'date',
                },
            }
            parameters.append(parameter)
        return parameters



# use pagination to modify how xml/rss renders
class XMLPostPagination(Pagination):
    def get_paginated_response_schema(self, schema):
        return {
            'type': 'string',
            'example': '<?xml version="1.0" ?>'
        }
    
    def get_paginated_response(self, data):
        return response.Response(data, headers={
            'rss_page_size': self.get_page_size(self.request),
            'rss_page_number': self.page.number,
            'rss_page_results_count': len(self.page),
            'rss_total_results_count': self.page.paginator.count,
        }, content_type="application/rss+xml; charset=UTF-8")
    
    def get_schema_operation_parameters(self, view):
        return super().get_schema_operation_parameters(view)

class RSSRenderer(renderers.BaseRenderer):
    media_type = "application/rss+xml"
    format = "xml"

    def render(self, data, accepted_media_type=None, renderer_context=None):
        return data


from datetime import UTC, datetime
import os
import time
from types import SimpleNamespace
import unittest, pytest
from urllib.parse import urljoin
from dateutil.parser import parse as parse_date

from tests.utils import remove_unknown_keys, wait_for_jobs

base_url = os.environ["SERVICE_BASE_URL"]
import requests
@pytest.mark.parametrize(
        ["feed_id", "metadata"],
        [
            ["d1d96b71-c687-50db-9d2b-d0092d1d163a", dict(title="updated title")],
            ["d63dad15-8e23-57eb-80f7-715cedf85f33", dict(title="updated title", description="new description")],
            ["d1d96b71-c687-50db-9d2b-d0092d1d163a", dict(pretty_url="https://muchdogesec.github.io/fakeblog123/?added_later=true")],
        ]
)
def test_update_feed_metadata(feed_id, metadata):
    resp = requests.patch(urljoin(base_url, f"api/v1/feeds/{feed_id}/"), json=metadata)
    assert resp.status_code == 201
    resp_data = resp.json()

    if expected_pretty_url := metadata.get("pretty_url"):
        assert resp_data["pretty_url"] == expected_pretty_url

    if expected_title := metadata.get("title"):
        assert resp_data["title"] == expected_title

    if expected_description := metadata.get("description"):
        assert resp_data["description"] == expected_description

# def test_feed_reindex(feed_id):
#     start_time = datetime.now(UTC)
#     resp = requests.patch(urljoin(base_url, f"api/v1/feeds/{feed_id}/"))
#     assert resp.status_code == 201
#     resp_data = resp.json()



import os
import time
from types import SimpleNamespace
import unittest, pytest
from urllib.parse import urljoin

from tests.utils import remove_unknown_keys, wait_for_jobs

base_url = os.environ["SERVICE_BASE_URL"]
import requests

def all_posts():
    DATA = [
        {
            "feed_id": "d63dad15-8e23-57eb-80f7-715cedf85f33",
            "title": "Example COM",
            "id": "223565cd-dd4f-54c2-9bbd-63019f39554f",
            "link": "https://example.com/",
            "pubdate": "2024-08-11T16:12:03Z",
            "author": "test",
            "categories": [
                "test",
                "test2"
            ]
        },
        {
            "feed_id": "d63dad15-8e23-57eb-80f7-715cedf85f33",
            "title": "Example ORG",
            "id": "a378c839-0940-56fb-b52c-e5b78d34ec94",
            "link": "https://example.org/",
            "pubdate": "2024-03-22T16:11:03Z",
            "author": "test",
            "categories": [
                "test",
                "test2"
            ]
        },
        {
            "feed_id": "d63dad15-8e23-57eb-80f7-715cedf85f33",
            "title": "Example COM under real",
            "id": "223565cd-dd4f-54c2-9bbd-63019f39554f",
            "link": "https://example.com/",
            "pubdate": "2024-08-11T16:12:03Z",
            "author": "test",
            "categories": [
                "test",
                "test2"
            ],
            "should_fail": True, #already added
        },
        {
            "feed_id": "d63dad15-8e23-57eb-80f7-715cedf85f33",
            "title": "Example ORG under real",
            "id": "a378c839-0940-56fb-b52c-e5b78d34ec94",
            "link": "https://example.org/",
            "pubdate": "2024-03-22T16:11:03Z",
            "author": "test",
            "categories": [
                "test",
                "test2"
            ],
            "should_fail": True, #already added
        },
    ]
    return [
        [d["feed_id"], d["link"], d, d.get("should_fail")]
            for d in DATA
    ]

@pytest.mark.parametrize(
    ["feed_id", "post_url", "post_data", "should_fail"],
    all_posts()
)
def test_add_post(feed_id, post_url, post_data, should_fail):
    payload = remove_unknown_keys(post_data, ["link", "title", "pubdate", "author", "categories"])
    post_job_resp = requests.post(urljoin(base_url, f"api/v1/feeds/{feed_id}/posts/"), json=dict(posts=[payload]))

    if should_fail:
        assert post_job_resp.status_code == 400, "add feed request expected to fail"
        return
    
    assert post_job_resp.status_code == 201, f"request failed: {post_job_resp.text}"
    post_job_resp_data = post_job_resp.json()
    assert post_job_resp_data["feed_id"] == feed_id, "wrong feed id"
    assert len(post_job_resp_data["urls"]["retrieving"]) == 1, "one post expected"
    post_id = post_job_resp_data["urls"]["retrieving"][0]["id"]
    expected_id = post_data["id"]
    assert post_id == expected_id
    job_id = post_job_resp_data['id']

    job_data = wait_for_jobs(job_id)
    post_data_resp = requests.get(urljoin(base_url, f"api/v1/posts/{post_id}/"))
    post_data_resp_data = post_data_resp.json()
    assert post_data_resp_data["title"] == post_data["title"]
    assert post_data_resp_data["pubdate"] == post_data["pubdate"]
    assert set(post_data_resp_data["categories"]) == set(post_data.get("categories", []))



from hypothesis.stateful import initialize
import schemathesis
import hooks

schema = schemathesis.from_uri("http://localhost:8006/api/schema/")




BaseAPIWorkflow = schema.as_state_machine()
BaseAPIWorkflow.run()


import os
import random
import time
from types import SimpleNamespace
import unittest, pytest
from urllib.parse import urljoin

from tests.utils import get_post_ids_for_job, is_sorted, remove_unknown_keys, wait_for_jobs

base_url = os.environ["SERVICE_BASE_URL"]
import requests


@pytest.mark.parametrize(
    ["filters", "expected_ids"],
    [
        [
            dict(feed_id="d1d96b71-c687-50db-9d2b-d0092d1d163a"),
            [
                "f8c75694-a834-5e35-b0a3-52034a1d9f6d",
                "85a762c9-00f9-5c0c-9858-498883e13ea1",
                "29be2407-d5d1-5b47-bbb5-1c51a84d48eb",
                "84a8ff1c-c463-5a97-b0c4-93daf7102b5f",
                "cfdb68b8-3d80-572d-9350-58baf57eabfb",
                "8f16d2be-7b06-5f3c-a851-9cce31b4fec8",
            ],
        ],  # feed does not exist
        [
            dict(link="test2/2024/08/07"),
            [
                "afef9ebd-2dee-5ab9-be0b-96c2ad83a1bb",
                "48310096-d1f3-5e30-9910-5d7d0fd400be",
                "d8aa9854-43fc-5816-b7ef-fc93810b29a5",
                "f8c75694-a834-5e35-b0a3-52034a1d9f6d",
            ],
        ],
        [
            dict(title="uPdATe this Post"),
            [
                "58514345-4e10-54c9-8f2c-d81507088079",
                "8c72f15c-abeb-5c90-b239-6429f53696f9",
                "8f16d2be-7b06-5f3c-a851-9cce31b4fec8",
                "f214c1fd-5370-5dff-bd49-fd74bf32c7fe",
            ],
        ],
        [
            dict(title="example org"),
            [
                "a378c839-0940-56fb-b52c-e5b78d34ec94",
            ],
        ],
        [
            dict(description="example domain"),
            [
                "223565cd-dd4f-54c2-9bbd-63019f39554f",
                "a378c839-0940-56fb-b52c-e5b78d34ec94",
            ],
        ],
    ],
)
def test_filters_generic(filters: dict, expected_ids: list[str]):
    expected_ids = set(expected_ids)
    url = urljoin(base_url, "api/v1/posts/")
    resp = requests.get(url, params=filters)
    resp_data = resp.json()
    assert resp_data["total_results_count"] == len(expected_ids)
    assert {post["id"] for post in resp_data["posts"]} == expected_ids


def random_posts_values(key, count):
    url = urljoin(base_url, "api/v1/posts/")
    resp = requests.get(url)
    data = resp.json()
    return [post[key] for post in random.choices(data["posts"], k=count)]


def more_pubdate_filters(count):
    filters = []
    pubdates = random_posts_values("pubdate", 50)
    for i in range(count):
        mmin = mmax = None
        if random.random() > 0.7:
            mmax = random.choice(pubdates)
        if random.random() < 0.3:
            mmin = random.choice(pubdates)
        if mmin or mmax:
            filters.append([mmin, mmax])
    return filters


@pytest.mark.parametrize(
    ["pubdate_min", "pubdate_max"],
    [
        ["2024-03-22T16:11:03Z", "2024-08-11T16:12:03Z"],
        ["2025-03-22T16:11:03Z", "2024-08-11T16:12:03Z"],
    ],
)
def test_pubdate_minmax(pubdate_min, pubdate_max):
    filters = {}
    if pubdate_min:
        filters.update(pubdate_min=pubdate_min)
    if pubdate_max:
        filters.update(pubdate_max=pubdate_max)

    assert pubdate_max or pubdate_min, "at least one of two filters required"

    url = urljoin(base_url, "api/v1/posts/")
    resp = requests.get(url, params=filters)
    assert resp.status_code == 200
    resp_data = resp.json()
    for d in resp_data["posts"]:
        if pubdate_min:
            assert (
                d["pubdate"] >= pubdate_min
            ), "pubdate must not be less than pubdate_min"
        if pubdate_max:
            assert (
                d["pubdate"] <= pubdate_max
            ), "pubdate must not be greater than pubdate_max"


@pytest.mark.parametrize(
    "updated_after", ["2024-03-22T16:11:03Z", "2030-03-22T16:11:03Z"]
)
def test_updated_after(updated_after):
    assert updated_after, "value cannot be None"

    url = urljoin(base_url, "api/v1/posts/")
    resp = requests.get(url, params=dict(pubdate_min=updated_after))
    assert resp.status_code == 200
    resp_data = resp.json()
    for d in resp_data["posts"]:
        assert (
            d["datetime_updated"] >= updated_after
        ), "datetime_updated must not be greater than updated_after"


def test_extra_updated_after(subtests):
    for datetime_updated in random_posts_values("datetime_updated", 12):
        with subtests.test(
            "randomly_generated updated_after query", updated_after=datetime_updated
        ):
            test_updated_after(datetime_updated)


def test_extra_pubdate_filters(subtests):
    for dmin, dmax in more_pubdate_filters(22):
        with subtests.test(
            "randomly_generated pubdate_* query", pubdate_min=dmin, pubdate_max=dmax
        ):
            test_pubdate_minmax(dmin, dmax)


def test_job_filter(subtests):
    def test_job_id_filter(job_id, post_ids):
        url = urljoin(base_url, "api/v1/posts/")
        resp = requests.get(url, params=dict(job_id=job_id))
        data = resp.json()
        for post in data["posts"]:
            assert post['id'] in post_ids, "post does not belong to job"
        assert data['total_results_count'] == len(post_ids)

    jobs_resp = requests.get(urljoin(base_url, "api/v1/jobs/"))
    for job in jobs_resp.json()['jobs']:
        with subtests.test("test_job_id_filter", job_id=job['id']):
            test_job_id_filter(job['id'], [x[0] for x in get_post_ids_for_job(job)])


@pytest.mark.parametrize(
        ["sort_filter", "expected_sort"],
        [
        ("", "pubdate_descending"), #default filter
        ("pubdate_descending", "pubdate_descending"),
        ("pubdate_ascending", "pubdate_ascending"),
        ("title_descending", "title_descending"),
        ("title_ascending", "title_ascending"),
        ("datetime_updated_descending", "datetime_updated_descending"),
        ("datetime_updated_ascending", "datetime_updated_ascending"),
        ("datetime_added_descending", "datetime_added_descending"),
        ("datetime_added_ascending", "datetime_added_ascending"),
    ]
)
def test_list_posts_sort(sort_filter: str, expected_sort: str):
    reports_url = urljoin(base_url, f"api/v1/posts/")
    filters = dict(sort=sort_filter) if sort_filter else None
    get_resp = requests.get(reports_url, params=filters)
    assert get_resp.status_code == 200, f"response: {get_resp.text}"
    posts = get_resp.json()["posts"]
    property, _, direction = expected_sort.rpartition('_')
    def sort_fn(obj):
        retval = obj[property]
        print(retval)
        return retval
    assert is_sorted(posts, key=sort_fn, reverse=direction == 'descending'), f"expected posts to be sorted by {property} in {direction} order"



import pytest
from django.urls import reverse
from rest_framework import status
from history4feed.app.models import Feed, Job, Post
from uuid import uuid4
from unittest.mock import MagicMock, patch
from datetime import datetime as dt

from history4feed.app.serializers import JobSerializer, PostSerializer
from history4feed.app.views import FeedPostView, feed_post_view

import pytest
from rest_framework import status
from history4feed.app.models import Post, Feed, Job
from datetime import datetime as dt
from unittest.mock import patch
from history4feed.app.serializers import PostWithFeedIDSerializer
from history4feed.app.views import FeedPostView
from dateutil.parser import parse as parse_date


from history4feed.app.utils import (
    DatetimeFieldUTC,
    Ordering,
    Pagination,
    MinMaxDateFilter,
)

# from .openapi_params import FEED_PARAMS, POST_PARAMS

from history4feed.app.serializers import PostWithFeedIDSerializer
from history4feed.app.models import Post, Feed, Job
from rest_framework import (
    status,
)
from django_filters.rest_framework import (
    DjangoFilterBackend,
)


@pytest.mark.django_db
def test_create_post_in_feed_success(client):
    feed = Feed.objects.create(url="https://example.com/rss.xml", title="Test Feed")
    job = Job.objects.create(feed=feed)

    with (
        patch.object(feed_post_view, "new_create_post_job") as mock_new_create_post_job,
        patch(
            "history4feed.app.views.JobSerializer", wraps=JobSerializer
        ) as mock_job_s_class,
    ):
        mock_new_create_post_job.return_value = job
        url = f"/api/v1/feeds/{feed.id}/posts/"

        payload = {
            "link": "https://example.com/test-post",
            "title": "Test Post",
            "pubdate": "2024-08-20T10:00:00.000Z",
        }

        resp = client.post(url, data=payload, content_type="application/json")
        assert mock_new_create_post_job.call_args[0][1] == feed.id
        assert mock_new_create_post_job.call_args[0][0].data == payload
        assert resp.status_code == status.HTTP_201_CREATED
        mock_job_s_class.assert_called_once_with(job)

        assert resp.data["id"] == str(job.id)
        assert resp.data["feed_id"] == str(job.feed.id)


@pytest.mark.django_db
def test_new_create_post_job():
    feed = Feed.objects.create(url="https://example.com/rss.xml", title="Test Feed")
    posts_data = [
        {
            "link": "https://example.com/test-post",
            "title": "Test Post",
            "pubdate": "2024-08-20T10:00:00.000Z",
        }
    ]
    payload = dict(posts=posts_data)
    request = MagicMock()
    request.data = payload
    with patch(
        "history4feed.app.views.task_helper.new_patch_posts_job"
    ) as mock_new_job:
        job = feed_post_view().new_create_post_job(request, feed.id)
        post = feed.posts.first()
        mock_new_job.assert_called_once_with(feed, [post])
        assert job == mock_new_job.return_value


@pytest.mark.django_db
def test_reindex_feed(client):
    feed = Feed.objects.create(url="https://example.com/rss.xml", title="Test Feed")
    p1 = Post.objects.create(feed=feed, title="First post", pubdate=dt.now())
    p2 = Post.objects.create(
        feed=feed,
        title="Second post",
        pubdate=dt.now(),
        link="https://example.net/post2",
    )
    job = Job.objects.create(feed=feed)
    with (
        patch("history4feed.app.views.task_helper.new_patch_posts_job") as mock_new_job,
        patch(
            "history4feed.app.views.JobSerializer",
            wraps=JobSerializer,
        ) as mock_job_s_class,
    ):
        mock_new_job.return_value = job
        resp = client.patch(f"/api/v1/feeds/{feed.id}/posts/reindex/")
        mock_new_job.assert_called_once_with(feed, (p1, p2))
        mock_job_s_class.assert_called_once_with(job)
        assert resp.data["id"] == str(job.id)
        assert resp.data["feed_id"] == str(job.feed.id)

